import pandas as pd
import __main__ as main
from multiprocessing.pool import ThreadPool
import numpy as np
import pickle
if not hasattr(main, '__file__'):
    from tqdm.notebook import tqdm
else:
    from tqdm import tqdm
class reconcilliation:
    def __init__(self,source_filepath:str,destination_filepath:str,fields_to_reconcile:list, source_delimiter=",",destination_delimiter=","):
        """Reconcile two csv files based on a list of fields

        Args:
            source_filepath (str): path to the source csv file
            destination_filepath (str): path to the destination csv file
            fields_to_reconcile (list): list of fields to reconcile, the first field in the list is the primary key
        """
        self.threads = 50
        self.source_filepath = source_filepath
        self.destination_filepath = destination_filepath
        self.fields_to_reconcile = fields_to_reconcile
        src = pd.read_csv(source_filepath,delimiter=source_delimiter)
        self.source_df = src
        dst = pd.read_csv(destination_filepath,delimiter=destination_delimiter)
        self.destination_df = dst
        self.reconcile()
        self.missing = pd.DataFrame(columns=self.source_df.columns)
        

    def reconcile(self,source_field_include=[],destination_field_include=[]):
        """Reconcile the source and destination csv files, creating a new dataframe with the reconciled data, 
        the new dataframe will have every other row be the source field and the destination field
        defines a count for number of exact matches from the source to the destination and the number or partial and exact matches
        """
        # create self.reconciled_df, every other column should be re-labled to source_FIELDNAME, destination_FIELDNAME,source_FIELDNAME2, destination_FIELDNAME2 ect.
        new_column_names = []
        self.matched_count = 0
        self.partial_count = 0
        self.missing_count = 0
        # merge dataframes based on the first field in the fields_to_reconcile list
        self.reconciled_df = pd.merge(self.source_df,self.destination_df,on=self.fields_to_reconcile[0],how="outer",indicator=True,suffixes=("","_destination"),)
        #add match_type as column
        self.reconciled_df["match_type"] = ""
        #compare fields in the fields_to_reconcile list, if they match set the match_type to exact, if they do not match set the match_type to partial
        for field in self.fields_to_reconcile[1:]:
            self.reconciled_df[field+"_is_match"] = np.where((self.reconciled_df[field] == self.reconciled_df[field+"_destination"]) and (self.reconciled_df["_merge"] == "both"),True,False)
            # self.reconciled_df[(self.reconciled_df[field] != self.reconciled_df[field+"_destination"]) and (self.reconciled_df["_merge"] == "both")]["match_type"] = "partial"
        
        #if all existing _is_match columns are True and the _merge column is both, set the match_type to exact
        #if any existing _is_match columns are False and the _merge column is both, set the match_type to partial
        self.reconciled_df.loc[(self.reconciled_df["_merge"] == "both") & (self.reconciled_df[[field+"_is_match" for field in self.fields_to_reconcile[1:]]].all(axis=1)),"match_type"] = "exact"
        self.reconciled_df.loc[(self.reconciled_df["_merge"] == "both") & (self.reconciled_df[[field+"_is_match" for field in self.fields_to_reconcile[1:]]].any(axis=1)),"match_type"] = "partial"
        #if the _merge column is left_only, set the match_type to missing
        self.reconciled_df.loc[self.reconciled_df["_merge"] == "left_only","match_type"] = "missing"

    def pickle_me(self,filepath):
        """pickle the reconciled dataframe

        Args:
            filepath (str): filepath to pickle the reconcilliation object
        """
        if filepath[-4:] != ".pkl":
            filepath = filepath+".pkl"
        with open(filepath,"wb") as f:
            pickle.dump(self,f)

    def _compare_row(self,row):
        """Compare a row from the reconciled dataframe

        Args:
            row (pandas.Series): row from the reconciled dataframe

        Returns:
            bool: True if the rows match, False if they do not
        """
        for field in self.fields_to_reconcile[1:]:
            source_field = field+"_source"
            destination_field = field+"_destination"
            if row[source_field] != row[destination_field]:
                return False
        return True
        
    def preview(self):
        """Preview the reconciled dataframe
        """
        return self.reconciled_df.head()




from distutils.core import setup

setup(
    name='ambra_ts_tools',
    version='"0.1.8"',
    author='Ethan York',
    description='Internal TS tools for Ambra Health utilizing public API',
    url='https://github.com/EthanYork/public-ambra-ts-scripts',
    packages=["ambra_ts_tools","ambra_ts_tools.Ambra_Clone","ambra_ts_tools.Ambra_Clone.V2"]
)


from csv import DictReader, DictWriter

from tkinter import filedialog
from tkinter import Tk
import tkinter as tk


def get_filenames(root):
    
    root.geometry=("500x200")
    ambra_label = tk.Label(root,text="Ambra File")
    source_label = tk.Label(root,text="Source File")
    def set_ambra_name():
        try:
            filename =  filedialog.askopenfile().name
            ambra_label.config(text=filename)
        except:
            pass
    def set_source_name():
        try:
            filename =  filedialog.askopenfile().name
            source_label.config(text=filename)
        except:
            pass
        
        
        
    ambra_button = tk.Button(root,text='Select Ambra File ("study_uid" & "image_count" MUST BE A COLUMN)',command=set_ambra_name)   
    source_button = tk.Button(root,text='Select Source File ("study_uid" & "image_count" MUST BE A COLUMN)',command=set_source_name)
    next_button = tk.Button(root,text='ok',command=root.quit,width=30)

    ambra_button.grid(column=1,row=1)
    ambra_label.grid(column=2,row=1)
    source_button.grid(column=1,row=2)
    source_label.grid(column=2,row=2)
    next_button.grid(column=1,row=4)
    root.mainloop()
    ambra = ambra_label.cget('text')
    source = source_label.cget('text')
    root.destroy()
    return (source,ambra)
def get_reader(root):
    filenames=None
    while filenames == None:
    #     get_filenames()
    #     # file_name = input()
    #     if file_name == "":
    #         return ""
    #     if file_name[-4:] != '.csv':
    #         file_name = file_name+".csv"
            
    #     if not path.isfile(getcwd()+"/"+file_name) and not path.isfile(file_name):
    #         print("'"+file_name+"' not found in '"+getcwd()+"'")
       filenames = get_filenames(root)
    

    ambra_file_name=filenames[1]
    source_file_name = filenames[0]
    print("Loading source file: "+source_file_name)
    source_file = open(source_file_name,'r',encoding="utf-8-sig")
    source_out = {val['study_uid'].strip():val for val in DictReader(source_file)}
    print("Study Rows in Source File: "+str(len(source_out)))
    print("Loading Ambra file "+ambra_file_name)
    ambra_file = open(ambra_file_name,'r',encoding="utf-8-sig")
    ambra_out = {val['study_uid'].strip():val for val in DictReader(ambra_file)}
    print("Study Rows in Ambra File: "+str(len(ambra_out)))
    return (source_out,ambra_out)

def create_output(readers):
    # try:
    source = readers[0]
    ambra = readers[1]
    no_match_uids = list(source.keys()-ambra.keys())
    present_uids = list(source.keys()-no_match_uids)
    image_mismatches = []
    for uid in present_uids:
        if source[uid]['image_count'] != ambra[uid]['image_count']:
            a = source[uid]['image_count']
            b = ambra[uid]['image_count']
            image_mismatches.append(uid)
    print("uids missing in ambra: "+str(len(no_match_uids)))
    print("image_count mismatches: "+str(len(image_mismatches)))
    # except Exception as E:
    #     print("ERROR matching study_uids", str(E))
    #     input("")
    # try:

    output_name = filedialog.asksaveasfilename()

    if output_name == "":
        output_name ="OUTPUT.csv"
    elif '.csv'not in output_name:
        output_name=output_name+".csv"

    output_file= open(output_name,"w+",encoding="utf-8-sig",newline="",)
    output_writer  = DictWriter(output_file,fieldnames=["study_uid","status"])
    output_writer.writeheader()
    for study_uid in no_match_uids:
        output_writer.writerow({"study_uid":study_uid,"status":"missing"})
    for study_uid in image_mismatches:
        output_writer.writerow({"study_uid":study_uid,"status":'ambra image_count: {0}, source image_count: {1}'.format(ambra[study_uid]["image_count"],source[study_uid]["image_count"])})
    output_file.close()
    # except Exception in E:
    #     print("ERROR writing output file", str(E))
    #     input("")
    input(output_name+" successfully created")

# try:

root = Tk()
readers = get_reader(root)
create_output(readers)

# except Exception as E:
#     print(str(E))
#     input("")

